#!/bin/zsh
#
# Copyright (C) 2009 Nick Bowler
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.  This file is offered as-is,
# without any warranty.
#
# Determine library dependencies for dynamically linked binaries and output
# directives suitable for gen_init_cpio to include all required libraries
# in an initramfs archive.  Users of this script will probably need to adjust
# the BASE, LDD and LOADER variables appropriately.

# Directory in which the libraries are located.
BASE=/usr/x86_64-pc-linux-uclibc

# How to run ldd.  The uClibc dynamic loader explodes wonderfully if
# LD_LIBRARY_PATH is not set correctly.
LDD="LD_LIBRARY_PATH=$BASE/lib:$BASE/usr/lib x86_64-pc-linux-uclibc-ldd"

# Where the dynamic loader is located.  Always included in the output.
LOADER="/lib/ld64-uClibc.so.0"

getlibs() {
	local file=$1

	if test -u "$1" -o -g "$1"; then
		file=`mktemp`
		cp -f "$1" "$file"
	fi

	eval "$LDD '$file'" | sed -n 's/.*=> \(.*\) (0x[[:xdigit:]]*)/\1/p'

	if [ "$file" != "$1" ]; then
		rm -f "$file"
	fi
}

libs=(`for i in "$@"; do getlibs "$i"; done | sort -u`)

output="file  $LOADER $BASE$LOADER 0755 0 0"

while [ $#libs -gt 0 ]; do
	i=${libs[1]}
	libs=(${libs[2,-1]})

	if ! [[ "$i" =~ "^$BASE" ]]; then continue; fi

	dest=${i#$BASE}

	if test -L "$i"; then
		output="$output\nslink $dest $(readlink "$i") 0644 0 0"
		next=`readlink "$i"`
		if [ "$next[0]" != "/" ]; then
			next="$(dirname "$i")/$next"
		fi
		libs+=$next
	elif test -f "$i"; then
		output="$output\nfile  $dest $i 0644 0 0"
	fi
done

echo "$output" | sort -u
